from ..responses import JsonResponse
from django.http.response import HttpResponse
from django.views.generic.base import View, TemplateResponseMixin
from django.views.generic.edit import FormMixin


class JsonView(View):
    response_class = JsonResponse

    def get(self, request, *args, **kwargs):
        return self.get_response(self.get_data(**kwargs))

    def get_data(self, **kwargs):
        return {
            'params': kwargs
        }

    def get_response(self, data, **response_kwargs):
        return self.response_class(data, **response_kwargs)


class AjaxResponseMixin(TemplateResponseMixin):
    status = True

    def render_to_response(self, context, **response_kwargs):
        if self.is_ajax():
            return self.get_ajax_response(context, **response_kwargs)
        return TemplateResponseMixin.render_to_response(self, context,
                                                        **response_kwargs)

    def serialize_context(self, context=None):
        return {'status': self.status}

    def get_ajax_response(self, context, **response_kwargs):
        return JsonResponse(self.serialize_context(context))

    def is_ajax(self):
        return self.request.is_ajax()


class AjaxFormMixin(AjaxResponseMixin, FormMixin):
    form_key_name = 'form'
    status = False

    def serialize_context(self, context=None):
        if not self.status:
            return {
                'status': False,
                'errors': context[self.form_key_name].errors
            }
        else:
            return {'status': True}

    def form_valid(self, form):
        if self.is_ajax():
            self.status = True
            return self.get_ajax_response({'form': form})
        return FormMixin.form_valid(self, form)

    def form_invalid(self, form):
        self.status = False
        return FormMixin.form_invalid(self, form)


class UpdateAjaxFormMixin(AjaxFormMixin):

    def form_valid(self, form):
        self.object = form.save()
        return AjaxFormMixin.form_valid(self, form)



from django.views.generic.base import TemplateView, View


def direct_to_template(request, template):
    return TemplateView.as_view(template_name=template)(request)


class ProxyView(View):
    '''
    Proxy to another views
    It allows to choose between some different view depends on conditions.
    Show different views for users from different groups for example.
    '''

    def dispatch(self, request, *args, **kwargs):
        return self.get_view(request, *args, **kwargs)

    def get_view(self, request, *args, **kwargs):
        raise NotImplementedError('You must override `get_view` method in '
                                  'child view')



from django.db.models.query import QuerySet, ValuesQuerySet
from django.http import HttpResponse
import datetime


class ExcelResponse(HttpResponse):

    def __init__(self, data, output_name='excel_data', headers=None,
                 force_csv=False, encoding='utf8'):

        # Make sure we've got the right type of data to work with
        valid_data = False
        if isinstance(data, ValuesQuerySet):
            data = list(data)
        elif isinstance(data, QuerySet):
            data = list(data.values())
        if hasattr(data, '__getitem__'):
            if isinstance(data[0], dict):
                if headers is None:
                    headers = data[0].keys()
                data = [[row[col] for col in headers] for row in data]
                data.insert(0, headers)
            if hasattr(data[0], '__getitem__'):
                valid_data = True
        assert valid_data is True, "ExcelResponse requires a sequence of sequences"

        import StringIO
        output = StringIO.StringIO()
        # Excel has a limit on number of rows; if we have more than that, make a csv
        use_xls = False
        if len(data) <= 65536 and force_csv is not True:
            try:
                import xlwt
            except ImportError:
                # xlwt doesn't exist; fall back to csv
                pass
            else:
                use_xls = True
        if use_xls:
            book = xlwt.Workbook(encoding=encoding)
            sheet = book.add_sheet('Sheet 1')
            styles = {'datetime': xlwt.easyxf(num_format_str='yyyy-mm-dd hh:mm:ss'),
                      'date': xlwt.easyxf(num_format_str='yyyy-mm-dd'),
                      'time': xlwt.easyxf(num_format_str='hh:mm:ss'),
                      'default': xlwt.Style.default_style}

            for rowx, row in enumerate(data):
                for colx, value in enumerate(row):
                    if isinstance(value, datetime.datetime):
                        cell_style = styles['datetime']
                    elif isinstance(value, datetime.date):
                        cell_style = styles['date']
                    elif isinstance(value, datetime.time):
                        cell_style = styles['time']
                    else:
                        cell_style = styles['default']
                    sheet.write(rowx, colx, value, style=cell_style)
            book.save(output)
            mimetype = 'application/vnd.ms-excel'
            file_ext = 'xls'
        else:
            for row in data:
                out_row = []
                for value in row:
                    if not isinstance(value, basestring):
                        value = unicode(value)
                    value = value.encode(encoding)
                    out_row.append(value.replace('"', '""'))
                output.write('"%s"\n' %
                             '","'.join(out_row))
            mimetype = 'text/csv'
            file_ext = 'csv'
        output.seek(0)
        super(ExcelResponse, self).__init__(content=output.getvalue(),
                                            mimetype=mimetype)
        self['Content-Disposition'] = 'attachment;filename="%s.%s"' % \
            (output_name.replace('"', '\"'), file_ext)



try:
    import json
except:
    from django.utils import simplejson as json
from django.utils.functional import Promise


class LazyEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, Promise):
            return unicode(o)
        else:
            return super(LazyEncoder, self).default(o)



from setuptools import setup, find_packages


version = __import__('extra_cbv').get_version()


def get_requires_list(filename):
    s = open(filename).read().split("\n")
    dependenses = []
    if len(s):
        for pkg in s:
            if pkg.strip() == '' or pkg.startswith("#"):
                continue
            if pkg.startswith("-e"):
                continue
                try:
                    p = pkg.split("#egg=")[1]
                    dependenses += [p, ]
                except:
                    continue
            else:
                dependenses += [pkg, ]
    return dependenses


setup(
    name = "django-extra-cbv",
    version = version,
    description = "Extra class based views for Django",
    keywords = "cbv, class based views",
    author = "Alex Kamedov",
    author_email = "alex@kamedov.ru",
    url = "https://github.com/alekam/django-extra-cbv",
    license = "New BSD License",
    platforms = ["any"],
    classifiers = ["Development Status :: 4 - Beta",
                   "Environment :: Web Environment",
                   "Framework :: Django",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: BSD License",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Topic :: Utilities"],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False
)
